#!/bin/sh

set -e

AWK=${AWK:-awk --lint=fatal}
MARKDOWN=${MARKDOWN:-pandoc -s -t man}

arg_mancat=-c
arg_mantitle=-t
arg_prgnam=-p
arg_version=-v

print_usage ()
{
	printf '%s %s MANCAT %s MANTITLE %s PRGNAM %s VERSION\n'\
		"$(basename $0)" "$arg_mancat" "$arg_mantitle"\
		"$arg_prgnam" "$arg_version"
	printf 'Generates a manual page with: %s\n' "$MARKDOWN"
	printf 'The following conversion is made post-generation:\n'
	printf '\t.TH PRGNAM "" "VERSION"\n'
	printf '\t->\n'
	printf '\t.TH PRGNAM MANCAT DATE VERSION "MANTITLE"\n'
	printf 'DATE (YYYY-MM-DD) is extracted from VERSION (YYMMDD)\n'
}

while	[ "$#" -ge 2 ]
do
	case "$1" in
		$arg_mancat)
			mancat="$2"
			;;
		$arg_mantitle)
			mantitle="$2"
			;;
		$arg_prgnam)
			prgnam="$2"
			;;
		$arg_version)
			version="$2"
			;;
		*)
			printf 'Unknown argument: %s\n' "$1"
			print_usage
			exit 1
			;;
	esac
	shift 2
done

if	[ -z "$mancat" ]
then
	printf 'Missing argument: %s\n' $arg_mancat
	print_usage
	exit 1
elif	[ -z "$mantitle" ]
then
	printf 'Missing argument: %s\n' $arg_mantitle
	print_usage
	exit 1
elif	[ -z "$prgnam" ]
then
	printf 'Missing argument: %s\n' $arg_prgnam
	print_usage
	exit 1
elif	[ -z "$version" ]
then
	printf 'Missing argument: %s\n' $arg_version
	print_usage
	exit 1
fi

$AWK '
BEGIN {
	skip		= 0
	prev		= ""
	cur		= ""

	print "% "'\"$prgnam\"'
	print "%"
	print "% "'\"$version\"'
}

{
	prev	= cur
	cur	= $0
	if	(skip > 0)
	{
		skip -= 1
	}
	gsub(//, "", cur)
}

/^===*/ {
	skip = 2
}

/^---*/ {
	prev = toupper(prev)
	gsub(/-/, "=", cur)
}

/^###### / {
	gsub(/######/, "#####", cur)
}

/^##### / {
	gsub(/#####/, "####", cur)
}

/^#### / {
	gsub(/####/, "###", cur)
}

/^### / {
	gsub(/###/, "##", cur)
}

{
	if	(NR > 1 && skip < 1)
	{
		print prev
	}
}

END {
	print cur
}
'\
| $MARKDOWN\
| $AWK '
function expand_date (d)
{
	return	"20"substr(d, 1, 2)"-"substr(d, 3, 2)"-"substr(d, 5, 2)
}

/^\.TH / {
	sub(/'$prgnam'/,	toupper('\"$prgnam\"'),	$0)
	sub(/""/,		'\"$mancat\"',		$0)
	sub(/"'$version'"/,
	expand_date('\"$version\"')" '$version' \"'"$mantitle"'\"",
		$0)
}

{
	gsub(/\\f\[C\]/, "\\f[CI]", $0)
	print $0
}
'

