/*
 * Copyright © 2013-2014 Alexander Söderlund (Sweden)
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
 * AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef HNEFATAFL_AIM
#define HNEFATAFL_AIM
#endif

#ifndef HNEF_AIM_CONFIG_H
#define HNEF_AIM_CONFIG_H

#ifndef HNEFATAFL_ZHASH
#undef HNEFATAFL_AIM_ZHASH
#endif

/*@unchecked@*/
/*@unused@*/
extern
const long HNEFATAFL_AIM_VERSION;

/*@in@*/
/*@notnull@*/
/*@observer@*/
/*@unused@*/
extern
const char *
hnef_aim_id (void)
/*@modifies nothing@*/
;

#endif

#ifndef HNEF_AIM_MINIMAXT_H
#define HNEF_AIM_MINIMAXT_H

#define	HNEF_AIM_DEPTHMAX_MAX	(unsigned short)1000

/*@unchecked@*/
/*@unused@*/
extern
const unsigned short	HNEF_AIM_DEPTHMAX_MIN,
			HNEF_AIM_DEPTHMAX_UNINIT;

/*@exposed@*/
struct hnef_aiminimax
{

	unsigned short		p_index;

	unsigned short		depth_max;

#ifdef	HNEFATAFL_AIM_ZHASH
/*@dependent@*/
/*@in@*/
/*@notnull@*/
	struct hnef_zhashtable	* tp_tab;
#endif	/* HNEFATAFL_AIM_ZHASH */

/*@in@*/
/*@notnull@*/
/*@owned@*/
	int			* board_escval;

/*@in@*/
/*@owned@*/
/*@notnull@*/
	struct hnef_listm	* opt_moves;

/*@in@*/
/*@owned@*/
/*@notnull@*/
	struct hnef_listmh	* opt_movehist;

/*@in@*/
/*@owned@*/
/*@notnull@*/
	struct hnef_listm	* * opt_buf_moves;

/*@in@*/
/*@owned@*/
/*@notnull@*/
	struct hnef_board	* * opt_buf_board;

	HNEF_BOOL		escape_ruleset;

	unsigned short		opt_buf_len;

};

#endif

#ifndef HNEF_AIM_MINIMAX_H
#define HNEF_AIM_MINIMAX_H

/*@unused@*/
extern
HNEF_BOOL
hnef_aiminimax_depth_max_valid (
	const unsigned short
	)
/*@modifies nothing@*/
;

/*@-protoparamname@*/
/*@null@*/
/*@only@*/
/*@partial@*/
/*@unused@*/
extern
struct hnef_aiminimax *
hnef_alloc_aiminimax (
/*@in@*/
/*@notnull@*/
	const struct hnef_game	* const game,
	const unsigned short	p_index,
	const unsigned short	depth_max,
/*@in@*/
/*@notnull@*/
	enum HNEF_FR		* const HNEF_RSTR fr
#ifdef	HNEFATAFL_AIM_ZHASH
	,
/*@dependent@*/
/*@in@*/
/*@null@*/
	struct hnef_zhashtable	* const old_tab,
	const size_t		mem_tab,
	const size_t		mem_col
#endif	/* HNEFATAFL_AIM_ZHASH */
	)
#ifdef	HNEFATAFL_AIM_ZHASH
/*@globals errno, internalState@*/
#else	/* HNEFATAFL_AIM_ZHASH */
/*@globals internalState@*/
#endif	/* HNEFATAFL_AIM_ZHASH */
/*@modifies * fr@*/
#ifdef	HNEFATAFL_AIM_ZHASH
/*@modifies errno, internalState@*/
#endif	/* HNEFATAFL_AIM_ZHASH */
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
extern
void
hnef_free_aiminimax (
/*@notnull@*/
/*@owned@*/
/*@special@*/
	struct hnef_aiminimax	* const aim
	)
/*@modifies aim@*/
/*@releases aim@*/
;
/*@=protoparamname@*/

#endif

#ifndef HNEF_AIM_ALGO_H
#define HNEF_AIM_ALGO_H

/*@-protoparamname@*/
/*@unused@*/
extern
enum HNEF_FR
aiminimax_command (
/*@in@*/
/*@notnull@*/
	const struct hnef_game	* const HNEF_RSTR g,
/*@in@*/
/*@notnull@*/
	struct hnef_aiminimax	* const aim,
/*@out@*/
/*@notnull@*/
	unsigned short		* pos,
/*@out@*/
/*@notnull@*/
	unsigned short		* dest,
/*@null@*/
	void			* data,
/*@null@*/
	enum HNEF_FR (* func_interrupt)
				(void *, HNEF_BOOL *, HNEF_BOOL *),
/*@null@*/
	enum HNEF_FR (* func_progress) (void *, int, int)
	)
/*@modifies * aim, * pos, * dest, * data@*/
;
/*@=protoparamname@*/

#endif

