/*
 * Copyright © 2013-2014 Alexander Söderlund (Sweden)
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
 * AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef HNEFATAFL_CORE
#define HNEFATAFL_CORE
#include "gleipnir_line.h"
#include "gleipnir_lines.h"
#include "gleipnir_lineread.h"
#endif

#ifndef HNEF_CORE_BOOL_T_H
#define HNEF_CORE_BOOL_T_H

typedef enum HNEF_BOOL_
{

	HNEF_FALSE	= 0,

	HNEF_TRUE	= 1

} HNEF_BOOL;

#endif

#ifndef HNEF_CORE_CONFIG_H
#define HNEF_CORE_CONFIG_H

#include <stddef.h>	/* size_t */
#include <stdio.h>	/* FILE */

#if	defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
#define	HNEF_RSTR restrict
#else
#define	HNEF_RSTR
#endif

/*@unchecked@*/
/*@unused@*/
extern
const long	HNEFATAFL_CORE_VERSION;

/*@in@*/
/*@notnull@*/
/*@observer@*/
/*@unused@*/
extern
const char *
hnef_core_id (void)
/*@modifies nothing@*/
;

/*@unused@*/
extern
size_t
hnef_core_license_arrc (void)
/*@*/
;

/*@in@*/
/*@notnull@*/
/*@observer@*/
/*@unused@*/
extern
const char * const *
hnef_core_license_arrv (void)
/*@*/
;

/*@-protoparamname@*/
/*@unused@*/
extern
HNEF_BOOL
hnef_core_license_print (
/*@in@*/
/*@notnull@*/
	FILE	* const HNEF_RSTR out
	)
/*@globals fileSystem@*/
/*@modifies fileSystem, * out@*/
;
/*@=protoparamname@*/

#endif

#ifndef HNEF_CORE_FUNC_T_H
#define HNEF_CORE_FUNC_T_H

enum HNEF_RVALID
{

	HNEF_RVALID_SUCCESS	= 0,

	HNEF_RVALID_FAIL_BITMASK,

	HNEF_RVALID_FAIL_BSIZE,

	HNEF_RVALID_FAIL_DTRAP_NOCAPT,

	HNEF_RVALID_FAIL_DTRAP_ONLYCOMPL,

	HNEF_RVALID_FAIL_DTRAP_NOSQUARES,

	HNEF_RVALID_FAIL_DTRAP_INCONSISTENT,

	HNEF_RVALID_FAIL_ESCAPE_NO_PIECE_XOR_SQUARE,

	HNEF_RVALID_FAIL_ID,

	HNEF_RVALID_FAIL_NAME,

	HNEF_RVALID_FAIL_NULL_PIECES,

	HNEF_RVALID_FAIL_NULL_SQUARES,

	HNEF_RVALID_FAIL_PIECE_BOARD_NONE,

	HNEF_RVALID_FAIL_PIECE_CAPTOWN,

	HNEF_RVALID_FAIL_PIECE_CAPTSIDES_CUST,

	HNEF_RVALID_FAIL_PIECE_CAPTSIDES_EDGE,

	HNEF_RVALID_FAIL_PIECE_ESCAPESTART,

	HNEF_RVALID_FAIL_PIECE_REFINOCCUP,

	HNEF_RVALID_FAIL_PIECE_STARTNOOCCUP,

	HNEF_RVALID_FAIL_PIECE_OWNER,

	HNEF_RVALID_FAIL_PIECE_CAPTSIDES,

	HNEF_RVALID_FAIL_PIECE_UNDEF,

	HNEF_RVALID_FAIL_PIECE_UNUSED,

	HNEF_RVALID_FAIL_SQUARE_CAPTSIDES,

	HNEF_RVALID_FAIL_SQUARE_CAPTSIDES_NOOCCUP,

	HNEF_RVALID_FAIL_SQUARE_CAPTSIDES_NOPIECE,

	HNEF_RVALID_FAIL_SQUARE_UNDEF,

	HNEF_RVALID_FAIL_SQUARE_UNUSED

};

enum HNEF_RREAD
{

	HNEF_RREAD_SUCCESS	= 0,

	HNEF_RREAD_FAIL_ARG_DUP,

	HNEF_RREAD_FAIL_ARG_EXCESS,

	HNEF_RREAD_FAIL_ARG_MISS,

	HNEF_RREAD_FAIL_ARG_UNK,

	HNEF_RREAD_FAIL_ARRB_PRESIZE,

	HNEF_RREAD_FAIL_ARRB_SIZE,

	HNEF_RREAD_FAIL_BIT_SINGLE,

	HNEF_RREAD_FAIL_NOOB,

	HNEF_RREAD_FAIL_NOPREL_BSIZE,

	HNEF_RREAD_FAIL_STRTOL,

	HNEF_RREAD_FAIL_TYPE_DUP,

	HNEF_RREAD_FAIL_TYPE_ORDER,

	HNEF_RREAD_FAIL_TYPE_OVERFLOW,

	HNEF_RREAD_FAIL_TYPE_UNDEF

};

enum HNEF_FR
{

	HNEF_FR_SUCCESS	= 0,

	HNEF_FR_FAIL_ALLOC,

	HNEF_FR_FAIL_GAME_REPLAY,

	HNEF_FR_FAIL_GAME_SAVE_MALF,

	HNEF_FR_FAIL_ILL_ARG,

	HNEF_FR_FAIL_ILL_STATE,

	HNEF_FR_FAIL_IO_FILE_CLOSE,

	HNEF_FR_FAIL_IO_FILE_OPEN,

	HNEF_FR_FAIL_IO_FILE_READ,

	HNEF_FR_FAIL_IO_FILE_WRITE,

	HNEF_FR_FAIL_IO_READ,

	HNEF_FR_FAIL_IO_WRITE,

	HNEF_FR_FAIL_MATH,

	HNEF_FR_FAIL_NULLPTR,

	HNEF_FR_FAIL_OTHER

};

#endif

#ifndef HNEF_CORE_NUMT_H
#define HNEF_CORE_NUMT_H

#include <stddef.h>	/* size_t */

/*@unchecked@*/
extern
const size_t	SIZET_MAX;

#endif

#ifndef HNEF_CORE_MOVE_T_H
#define HNEF_CORE_MOVE_T_H

/*@unchecked@*/
extern
const unsigned short	HNEF_BOARDPOS_NONE;

/*@concrete@*/
struct hnef_move
{

	unsigned short	pos,
			dest;

};

#endif

#ifndef HNEF_CORE_MOVEH_T_H
#define HNEF_CORE_MOVEH_T_H

/*@concrete@*/
struct hnef_moveh
{

	HNEF_BOOL	irrev;

	unsigned short	pos,
			dest;

};

#endif

#ifndef HNEF_CORE_LISTM_T_H
#define HNEF_CORE_LISTM_T_H

/*@concrete@*/
struct hnef_listm
{

/*@notnull@*/
/*@owned@*/
	struct hnef_move	* elems;

	size_t		elemc,
			capc;

};

#endif

#ifndef HNEF_CORE_LISTMH_T_H
#define HNEF_CORE_LISTMH_T_H

#include <stdlib.h>	/* size_t */

/*@unchecked@*/
extern
const size_t	HNEF_LISTMH_CAP_DEF;

/*@concrete@*/
struct hnef_listmh
{

/*@notnull@*/
/*@owned@*/
	struct hnef_moveh	* elems;

	size_t			elemc,
				capc;

};

#endif

#ifndef HNEF_CORE_BOARD_T_H
#define HNEF_CORE_BOARD_T_H

typedef unsigned char	HNEF_BIT_U8;

/*@unchecked@*/
extern
const HNEF_BIT_U8	HNEF_BIT_U8_EMPTY;

/*@exposed@*/
struct hnef_board
{

/*@notnull@*/
/*@owned@*/
	HNEF_BIT_U8	* pieces;

	unsigned short	turn;

	unsigned short	opt_captloss_piece_lastseen;

};

#endif

#ifndef HNEF_CORE_PLAYER_T_H
#define HNEF_CORE_PLAYER_T_H

/*@unchecked@*/
extern
const unsigned short	HNEF_PLAYERS_MAX;

/*@unchecked@*/
extern
const unsigned short	HNEF_PLAYER_UNINIT;

/*@exposed@*/
struct hnef_player
{

	unsigned short	index;

	HNEF_BIT_U8	opt_owned,
			opt_owned_esc;

};

#endif

#ifndef HNEF_CORE_TYPES_H
#define HNEF_CORE_TYPES_H

#define HNEF_TYPE_MAX (unsigned short)8

#define HNEF_TYPE_INDEX_INVALID (unsigned short)8

/*@unchecked@*/
extern
const unsigned short	HNEF_TPIECE_CAPT_SIDES_MAX,
			HNEF_TPIECE_CAPT_SIDES_MIN,
			HNEF_TPIECE_CAPT_SIDES_DEF;

extern
unsigned short
hnef_type_index_get (
	const HNEF_BIT_U8
	)
/*@modifies nothing@*/
;

#endif

#ifndef HNEF_CORE_GAME_T_H
#define HNEF_CORE_GAME_T_H

struct hnef_game
{

/*@notnull@*/
/*@owned@*/
	struct hnef_ruleset	* rules;

/*@null@*/
/*@owned@*/
	struct hnef_board	* board;

/*@notnull@*/
/*@owned@*/
	struct hnef_player	* * players;

	unsigned short		playerc;

/*@notnull@*/
/*@owned@*/
	struct hnef_listmh	* movehist,
				* opt_replayhist;

/*@notnull@*/
/*@owned@*/
	struct gleip_lines	* opt_filereadline;

	HNEF_BOOL		opt_valid_known;

	enum HNEF_RVALID	opt_valid_code;

};

#endif

#ifndef HNEF_CORE_TYPE_PIECE_T_H
#define HNEF_CORE_TYPE_PIECE_T_H

/*@exposed@*/
struct hnef_type_piece
{

	HNEF_BIT_U8	bit,
			captures,
			dbl_trap_squares,
			noreturn,
			occupies,
			ui_bit;

	unsigned short	owner,
			capt_sides;

	HNEF_BOOL	capt_edge,
			capt_loss,
			custodial,
			dbl_trap,
			dbl_trap_capt,
			dbl_trap_compl,
			dbl_trap_edge,
			dbl_trap_encl,
			escape;

};

#endif

#ifndef HNEF_CORE_TYPE_SQUARE_T_H
#define HNEF_CORE_TYPE_SQUARE_T_H

/*@exposed@*/
struct hnef_type_square
{

	HNEF_BIT_U8	bit,
			captures,
			capt_sides_pieces,
			ui_bit;

	unsigned short	capt_sides;

	HNEF_BOOL	escape;

};

#endif

#ifndef HNEF_CORE_RULESET_T_H
#define HNEF_CORE_RULESET_T_H

/*@exposed@*/
struct hnef_ruleset
{

/*@null@*/
/*@owned@*/
	char	* id,
		* name,
		* ui_theme;

	HNEF_BOOL	forbid_repeat;

	unsigned short	bheight,
			bwidth,
			type_piecec,
			type_squarec,
			opt_blen,
			opt_escape_piecec;

/*@notnull@*/
/*@owned@*/
	unsigned short	* opt_squares_escape;

/*@null@*/
/*@owned@*/
	HNEF_BIT_U8	* pieces,
			* squares;

	HNEF_BIT_U8	opt_tp_captloss,
			opt_tp_dbl_trap,
			opt_tp_dbl_trap_capt,
			opt_tp_dbl_trap_compl,
			opt_tp_escape,
			opt_ts_capt_sides,
			opt_ts_escape;

/*@notnull@*/
	struct hnef_type_piece	type_pieces[(unsigned short)
						(HNEF_TYPE_MAX + 1)];

/*@notnull@*/
	struct hnef_type_square	type_squares[(unsigned short)
						(HNEF_TYPE_MAX + 1)];

};

#endif

#ifndef HNEF_CORE_FUNC_H
#define HNEF_CORE_FUNC_H

extern
HNEF_BOOL
hnef_rread_good (
	const enum HNEF_RREAD
	)
/*@modifies nothing@*/
;

extern
HNEF_BOOL
hnef_rvalid_good (
	const enum HNEF_RVALID
	)
/*@modifies nothing@*/
;

extern
HNEF_BOOL
hnef_fr_good (
	const enum HNEF_FR
	)
/*@modifies nothing@*/
;

/*@unused@*/
extern
HNEF_BOOL
hnef_fr_fail_io (
	const enum HNEF_FR
	)
/*@modifies nothing@*/
;

#endif

#ifndef HNEF_CORE_BOARD_H
#define HNEF_CORE_BOARD_H

#ifdef HNEFATAFL_ZHASH
#endif

/*@-protoparamname@*/
/*@unused@*/
extern
void
hnef_board_copy (
/*@in@*/
/*@notnull@*/
	const struct hnef_board	* const HNEF_RSTR src,
/*@in@*/
/*@notnull@*/
	struct hnef_board	* const HNEF_RSTR dest,
	const unsigned short	blen
	)
/*@modifies * dest@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
extern
enum HNEF_FR
hnef_board_move_unsafe (
/*@in@*/
/*@notnull@*/
	const struct hnef_game		* const HNEF_RSTR game,
/*@in@*/
/*@notnull@*/
	struct hnef_board		* const board,
/*@in@*/
/*@notnull@*/
	struct hnef_listmh		* const movehist,
	const unsigned short		pos,
	const unsigned short		dest
#ifdef HNEFATAFL_ZHASH
	,
/*@in@*/
/*@null@*/
	const struct hnef_zhashtable	* const HNEF_RSTR hashtable,
/*@in@*/
/*@null@*/
	unsigned int			* const hashkey,
/*@in@*/
/*@null@*/
	unsigned int			* const hashlock
#endif
	)
/*@modifies * board, * movehist@*/
#ifdef HNEFATAFL_ZHASH
/*@modifies * hashkey, * hashlock@*/
#endif
;
/*@=protoparamname@*/

/*@-protoparamname@*/
extern
enum HNEF_FR
hnef_game_move (
/*@in@*/
/*@notnull@*/
	struct hnef_game	* const HNEF_RSTR game,
	const unsigned short	pos,
	const unsigned short	dest,
/*@out@*/
/*@notnull@*/
	HNEF_BOOL		* const HNEF_RSTR legal
	)
/*@modifies * game, * legal@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
extern
enum HNEF_FR
hnef_board_moves_get_pos (
/*@in@*/
/*@notnull@*/
	const struct hnef_game		* const HNEF_RSTR game,
/*@in@*/
/*@notnull@*/
	const struct hnef_board		* const HNEF_RSTR board,
/*@in@*/
/*@notnull@*/
	const struct hnef_listmh	* const HNEF_RSTR movehist,
/*@in@*/
/*@notnull@*/
	struct hnef_listm		* const list,
	const unsigned short		pos_piece
	)
/*@modifies * list@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
extern
enum HNEF_FR
hnef_board_moves_get (
/*@in@*/
/*@notnull@*/
	const struct hnef_game		* const HNEF_RSTR game,
/*@in@*/
/*@notnull@*/
	const struct hnef_board		* const HNEF_RSTR board,
/*@in@*/
/*@notnull@*/
	const struct hnef_listmh	* const HNEF_RSTR movehist,
/*@in@*/
/*@notnull@*/
	struct hnef_listm		* const list
	)
/*@modifies * list@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
extern
enum HNEF_FR
hnef_game_moves_get_pos (
/*@in@*/
/*@notnull@*/
	const struct hnef_game		* const HNEF_RSTR game,
/*@in@*/
/*@notnull@*/
	struct hnef_listm		* const list,
	const unsigned short		pos_piece
	)
/*@modifies * list@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
extern
int
hnef_board_movec_get (
/*@in@*/
/*@notnull@*/
	const struct hnef_game		* const HNEF_RSTR game,
/*@in@*/
/*@notnull@*/
	struct hnef_board		* const HNEF_RSTR board,
/*@in@*/
/*@notnull@*/
	const struct hnef_listmh	* const HNEF_RSTR movehist,
	const HNEF_BIT_U8		piecemask,
	const HNEF_BOOL			fast
	)
/*@modifies * board@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
HNEF_BOOL
hnef_board_game_over (
/*@in@*/
/*@notnull@*/
	const struct hnef_game		* const HNEF_RSTR game,
/*@in@*/
/*@notnull@*/
	struct hnef_board		* const HNEF_RSTR board,
/*@in@*/
/*@notnull@*/
	const struct hnef_listmh	* const HNEF_RSTR movehist,
/*@in@*/
/*@notnull@*/
	unsigned short			* const winner
	)
/*@modifies * board, * winner@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
HNEF_BOOL
hnef_game_over (
/*@in@*/
/*@notnull@*/
	struct hnef_game	* const HNEF_RSTR game,
/*@in@*/
/*@notnull@*/
	unsigned short		* const winner
	)
/*@modifies * game, * winner@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
extern
void
hnef_board_init (
	struct hnef_board	* const board,
	const unsigned short	blen
	)
/*@modifies board->turn, board->pieces@*/
/*@modifies board->opt_captloss_piece_lastseen@*/
;
/*@=protoparamname@*/

/*@null@*/
/*@only@*/
/*@partial@*/
extern
struct hnef_board *
hnef_alloc_board (
	const unsigned short
	)
/*@modifies nothing@*/
;

/*@-protoparamname@*/
extern
void
hnef_free_board (
/*@notnull@*/
/*@owned@*/
/*@special@*/
	struct hnef_board	* const HNEF_RSTR board
	)
/*@modifies board->pieces, board@*/
/*@releases board->pieces, board@*/
;
/*@=protoparamname@*/

#endif

#ifndef HNEF_CORE_PLAYER_H
#define HNEF_CORE_PLAYER_H

/*@-protoparamname@*/
extern
void
hnef_player_initopt (
/*@in@*/
/*@notnull@*/
	const struct hnef_game	* const HNEF_RSTR game,
/*@in@*/
/*@notnull@*/
	struct hnef_player	* const HNEF_RSTR player
	)
/*@modifies * player@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
extern
void
hnef_player_clear (
/*@in@*/
/*@notnull@*/
	struct hnef_player	* const HNEF_RSTR player
	)
/*@modifies * player@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
extern
HNEF_BOOL
hnef_player_init (
	struct hnef_player	* const HNEF_RSTR player,
	const unsigned short	index
	)
/*@modifies * player@*/
;
/*@=protoparamname@*/

/*@unused@*/
extern
HNEF_BOOL
hnef_player_index_valid (
	const unsigned short
	)
/*@modifies nothing@*/
;

/*@null@*/
/*@only@*/
/*@partial@*/
extern
struct hnef_player *
hnef_alloc_player (void)
/*@modifies nothing@*/
;

/*@-protoparamname@*/
extern
void
hnef_free_player (
/*@notnull@*/
/*@owned@*/
/*@special@*/
	struct hnef_player	* const player
	)
/*@modifies player@*/
/*@releases player@*/
;
/*@=protoparamname@*/

#endif

#ifndef HNEF_CORE_GAME_H
#define HNEF_CORE_GAME_H

/*@-protoparamname@*/
extern
void
hnef_game_reset_board (
/*@in@*/
/*@notnull@*/
	struct hnef_game	* const HNEF_RSTR game
	)
/*@modifies game->board@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
extern
void
hnef_game_reset (
/*@in@*/
/*@notnull@*/
	struct hnef_game	* const HNEF_RSTR game
	)
/*@modifies * game@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
extern
enum HNEF_FR
hnef_game_init_done (
/*@in@*/
/*@notnull@*/
	struct hnef_game	* const HNEF_RSTR game
	)
/*@modifies * game@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
extern
void
hnef_game_clear (
/*@notnull@*/
/*@partial@*/
	struct hnef_game	* const HNEF_RSTR game
	)
/*@modifies * game@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
extern
HNEF_BOOL
hnef_game_rules_equal (
/*@in@*/
/*@notnull@*/
	struct hnef_game	* const HNEF_RSTR g1,
/*@in@*/
/*@notnull@*/
	struct hnef_game	* const HNEF_RSTR g2
	)
/*@modifies * g1, * g2@*/
;
/*@=protoparamname@*/

/*@null@*/
/*@only@*/
/*@partial@*/
/*@unused@*/
extern
struct hnef_game *
hnef_alloc_game (void)
/*@modifies nothing@*/
;

/*@-protoparamname@*/
/*@unused@*/
extern
void
hnef_free_game (
/*@notnull@*/
/*@owned@*/
/*@special@*/
	struct hnef_game	* const HNEF_RSTR game
	)
/*@modifies * game, game@*/
/*@releases game@*/
;
/*@=protoparamname@*/

#endif

#ifndef HNEF_CORE_GAMEUTIL_H
#define HNEF_CORE_GAMEUTIL_H

#include <stdio.h>	/* FILE */

/*@unused@*/
extern
HNEF_BIT_U8
hnef_game_uibit_piece (
/*@notnull@*/
	const struct hnef_type_piece	* const HNEF_RSTR
	)
/*@modifies nothing@*/
;

/*@unused@*/
extern
HNEF_BIT_U8
hnef_game_uibit_square (
/*@notnull@*/
	const struct hnef_type_square	* const HNEF_RSTR
	)
/*@modifies nothing@*/
;

/*@-protoparamname@*/
/*@unused@*/
extern
enum HNEF_FR
hnef_game_save_file (
/*@in@*/
/*@notnull@*/
	struct hnef_game	* const g,
/*@in@*/
/*@notnull@*/
	FILE			* const file
	)
/*@globals errno, fileSystem@*/
/*@modifies errno, fileSystem, * g, file@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
extern
enum HNEF_FR
hnef_game_save (
/*@in@*/
/*@notnull@*/
	struct hnef_game	* const g,
/*@in@*/
/*@notnull@*/
	const char		* const filename
	)
/*@globals errno, fileSystem@*/
/*@modifies errno, fileSystem, * g@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
extern
enum HNEF_FR
hnef_game_load (
/*@in@*/
/*@notnull@*/
	struct hnef_game	* const g,
/*@in@*/
/*@notnull@*/
	const char		* const filename,
/*@out@*/
/*@notnull@*/
	HNEF_BOOL		* const id_good
	)
/*@globals errno, fileSystem@*/
/*@modifies errno, fileSystem, * g, id_good@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
extern
enum HNEF_FR
hnef_game_undo (
/*@in@*/
/*@notnull@*/
	struct hnef_game	* const g,
/*@out@*/
/*@notnull@*/
	HNEF_BOOL		* const success,
	const size_t		undoc
	)
/*@modifies * g, * success@*/
;
/*@=protoparamname@*/

#endif

#ifndef HNEF_CORE_LISTM_H
#define HNEF_CORE_LISTM_H

/*@null@*/
/*@only@*/
/*@partial@*/
/*@unused@*/
extern
struct hnef_listm *
hnef_alloc_listm (
	const size_t
	)
/*@modifies nothing@*/
;

/*@-protoparamname@*/
/*@unused@*/
extern
void
hnef_free_listm (
/*@notnull@*/
/*@owned@*/
/*@special@*/
	struct hnef_listm	* const HNEF_RSTR list
	)
/*@modifies list->elems, list@*/
/*@releases list->elems, list@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
extern
enum HNEF_FR
hnef_listm_add (
/*@notnull@*/
/*@partial@*/
	struct hnef_listm	* const list,
	const unsigned short	pos,
	const unsigned short	dest
	)
/*@modifies * list@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
extern
void
hnef_listm_clear (
/*@notnull@*/
	struct hnef_listm	* const HNEF_RSTR list
	)
/*@modifies * list@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
extern
enum HNEF_FR
hnef_listm_swaptofr (
	struct hnef_listm	* const HNEF_RSTR list,
	const size_t		index
	)
/*@modifies * list@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
extern
enum HNEF_FR
hnef_listm_remove (
	struct hnef_listm	* const HNEF_RSTR list,
	const size_t		index
	)
/*@modifies * list@*/
;
/*@=protoparamname@*/

#endif

#ifndef HNEF_CORE_LISTMH_H
#define HNEF_CORE_LISTMH_H

/*@null@*/
/*@only@*/
/*@partial@*/
extern
struct hnef_listmh *
hnef_alloc_listmh (
	const size_t
	)
/*@modifies nothing@*/
;

/*@-protoparamname@*/
extern
void
hnef_free_listmh (
/*@notnull@*/
/*@owned@*/
/*@special@*/
	struct hnef_listmh	* const HNEF_RSTR list
	)
/*@modifies list->elems, list@*/
/*@releases list->elems, list@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
extern
enum HNEF_FR
hnef_listmh_add (
/*@notnull@*/
/*@partial@*/
	struct hnef_listmh	* const list,
	const unsigned short	pos,
	const unsigned short	dest,
	const HNEF_BOOL		irrev
	)
/*@modifies * list@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
extern
enum HNEF_FR
hnef_listmh_copy (
/*@in@*/
/*@notnull@*/
	const struct hnef_listmh	* const HNEF_RSTR src,
/*@in@*/
/*@notnull@*/
	struct hnef_listmh		* const HNEF_RSTR dest
	)
/*@modifies * dest@*/
;
/*@=protoparamname@*/

#endif

#ifndef HNEF_CORE_NUM_H
#define HNEF_CORE_NUM_H

/*@unused@*/
extern
int
hnef_max_int (
	const int,
	const int
	)
/*@modifies nothing@*/
;

/*@unused@*/
extern
int
hnef_min_int (
	const int,
	const int
	)
/*@modifies nothing@*/
;

extern
unsigned short
hnef_max_ushrt (
	const unsigned short,
	const unsigned short
	)
/*@modifies nothing@*/
;

extern
unsigned short
hnef_min_ushrt (
	const unsigned short,
	const unsigned short
	)
/*@modifies nothing@*/
;

extern
HNEF_BOOL
hnef_single_bit (
	const unsigned int
	)
/*@modifies nothing@*/
;

/*@-protoparamname@*/
/*@unused@*/
extern
HNEF_BOOL
hnef_texttoint (
/*@in@*/
/*@notnull@*/
	const char	* const HNEF_RSTR str_num,
/*@out@*/
/*@notnull@*/
	int		* const HNEF_RSTR num
	)
/*@globals errno@*/
/*@modifies errno, * num@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
extern
HNEF_BOOL
hnef_texttoushort (
/*@in@*/
/*@notnull@*/
	const char	* const HNEF_RSTR str_num,
/*@out@*/
/*@notnull@*/
	unsigned short	* const HNEF_RSTR num
	)
/*@globals errno@*/
/*@modifies errno, * num@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
extern
HNEF_BOOL
hnef_texttosize (
/*@in@*/
/*@notnull@*/
	const char	* const HNEF_RSTR str_num,
/*@out@*/
/*@notnull@*/
	size_t *	const HNEF_RSTR num
	)
/*@globals errno@*/
/*@modifies errno, * num@*/
;
/*@=protoparamname@*/

#endif

#ifndef HNEF_CORE_RREAD_H
#define HNEF_CORE_RREAD_H

/*@-protoparamname@*/
/*@unused@*/
extern
enum HNEF_FR
hnef_game_parse_line_tokens (
/*@in@*/
/*@notnull@*/
	struct hnef_game * const HNEF_RSTR game,
/*@in@*/
/*@notnull@*/
	const char	* const HNEF_RSTR * const HNEF_RSTR wordv,
	const size_t	wordc,
/*@in@*/
/*@notnull@*/
	enum HNEF_RREAD	* const HNEF_RSTR fail_read
	)
/*@globals errno@*/
/*@modifies errno, * game, * fail_read@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
extern
enum HNEF_FR
hnef_game_read_line_single (
/*@in@*/
/*@notnull@*/
	struct hnef_game		* const HNEF_RSTR game,
/*@in@*/
/*@notnull@*/
	const struct gleip_lines	* const GLEIP_RSTR line,
/*@in@*/
/*@notnull@*/
	enum HNEF_RREAD			* const HNEF_RSTR fail_read
	)
/*@globals errno@*/
/*@modifies errno, * game, * fail_read@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
extern
enum HNEF_FR
hnef_game_read_file (
/*@in@*/
/*@notnull@*/
	struct hnef_game	* const HNEF_RSTR game,
/*@in@*/
/*@notnull@*/
	FILE			* const HNEF_RSTR file,
/*@in@*/
/*@notnull@*/
	struct gleip_lines	* const GLEIP_RSTR line,
/*@in@*/
/*@notnull@*/
	enum HNEF_RREAD		* const HNEF_RSTR fail_read
	)
/*@globals errno@*/
/*@modifies errno, * game, * file, * line, * fail_read@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
extern
enum HNEF_FR
hnef_game_read_line (
/*@in@*/
/*@notnull@*/
	struct hnef_game	* const HNEF_RSTR game,
/*@in@*/
/*@notnull@*/
	const char		* const content,
/*@in@*/
/*@notnull@*/
	struct gleip_lines	* const GLEIP_RSTR line,
/*@in@*/
/*@notnull@*/
	enum HNEF_RREAD		* const HNEF_RSTR fail_read
	)
/*@globals errno@*/
/*@modifies errno, * game, * line, * fail_read@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
extern
enum HNEF_FR
hnef_game_init_file (
/*@in@*/
/*@notnull@*/
	struct hnef_game	* const HNEF_RSTR game,
/*@in@*/
/*@notnull@*/
	FILE			* const HNEF_RSTR file,
/*@in@*/
/*@notnull@*/
	struct gleip_lines	* const GLEIP_RSTR line,
/*@in@*/
/*@notnull@*/
	enum HNEF_RREAD		* const HNEF_RSTR fail_read,
/*@in@*/
/*@notnull@*/
	enum HNEF_RVALID	* const HNEF_RSTR fail_valid
	)
/*@globals errno@*/
/*@modifies errno, * game, * file, * line, * fail_read, * fail_valid@*/
;
/*@=protoparamname@*/

/*@-protoparamname@*/
/*@unused@*/
extern
enum HNEF_FR
hnef_game_init_line (
/*@in@*/
/*@notnull@*/
	struct hnef_game	* const HNEF_RSTR game,
/*@in@*/
/*@notnull@*/
	const char		* const content,
/*@in@*/
/*@notnull@*/
	struct gleip_lines	* const GLEIP_RSTR line,
/*@in@*/
/*@notnull@*/
	enum HNEF_RREAD		* const HNEF_RSTR fail_read,
/*@in@*/
/*@notnull@*/
	enum HNEF_RVALID	* const HNEF_RSTR fail_valid
	)
/*@globals errno@*/
/*@modifies errno, * game, * line, * fail_read, * fail_valid@*/
;
/*@=protoparamname@*/

#endif

#ifndef HNEF_CORE_RVALID_H
#define HNEF_CORE_RVALID_H

/*@-protoparamname@*/
extern
enum HNEF_RVALID
hnef_game_valid (
/*@in@*/
/*@notnull@*/
	struct hnef_game	* const HNEF_RSTR game
	)
/*@modifies game->opt_valid_known, game->opt_valid_code@*/
;
/*@=protoparamname@*/

#endif

#ifndef HNEF_CORE_TYPE_PIECE_H
#define HNEF_CORE_TYPE_PIECE_H

extern
HNEF_BOOL
hnef_type_piece_can_capture (
/*@in@*/
/*@notnull@*/
	const struct hnef_type_piece	* const HNEF_RSTR,
	const HNEF_BIT_U8
	)
/*@modifies nothing@*/
;

extern
HNEF_BOOL
hnef_type_piece_can_occupy (
/*@in@*/
/*@notnull@*/
	const struct hnef_type_piece	* const HNEF_RSTR,
	const HNEF_BIT_U8
	)
/*@modifies nothing@*/
;

extern
HNEF_BOOL
hnef_type_piece_can_return (
/*@in@*/
/*@notnull@*/
	const struct hnef_type_piece	* const HNEF_RSTR,
	const HNEF_BIT_U8,
	const HNEF_BIT_U8
	)
/*@modifies nothing@*/
;

/*@-protoparamname@*/
extern
void
hnef_type_piece_init (
/*@in@*/
/*@notnull@*/
	struct hnef_type_piece		* const HNEF_RSTR tp
	)
/*@modifies * tp@*/
;
/*@=protoparamname@*/

/*@in@*/
/*@notnull@*/
extern
struct hnef_type_piece *
hnef_type_piece_get (
/*@in@*/
/*@notnull@*/
/*@returned@*/
	struct hnef_ruleset	* const HNEF_RSTR,
	const HNEF_BIT_U8
	)
/*@modifies nothing@*/
;

/*@-protoparamname@*/
extern
enum HNEF_FR
hnef_type_piece_set (
/*@in@*/
/*@notnull@*/
	struct hnef_ruleset	* const HNEF_RSTR rules,
	const HNEF_BIT_U8	bit
	)
/*@modifies * rules@*/
;
/*@=protoparamname@*/

#endif

#ifndef HNEF_CORE_TYPE_SQUARE_T
#define HNEF_CORE_TYPE_SQUARE_T

/*@unchecked@*/
extern
unsigned short HNEF_TSQUARE_CAPT_SIDES_NONE;

extern
HNEF_BOOL
hnef_type_square_can_capture (
/*@in@*/
/*@notnull@*/
	const struct hnef_type_square	* const HNEF_RSTR,
	const HNEF_BIT_U8
	)
/*@modifies nothing@*/
;

/*@-protoparamname@*/
extern
void
hnef_type_square_init (
/*@in@*/
/*@notnull@*/
	struct hnef_type_square	* const HNEF_RSTR ts
	)
/*@modifies * ts@*/
;
/*@=protoparamname@*/

/*@in@*/
/*@notnull@*/
extern
struct hnef_type_square *
hnef_type_square_get (
/*@in@*/
/*@notnull@*/
/*@returned@*/
	struct hnef_ruleset * const HNEF_RSTR,
	const HNEF_BIT_U8
	)
/*@modifies nothing@*/
;

/*@-protoparamname@*/
extern
enum HNEF_FR
hnef_type_square_set (
/*@in@*/
/*@notnull@*/
	struct hnef_ruleset	* const HNEF_RSTR rules,
	const HNEF_BIT_U8	bit
	)
/*@modifies * rules@*/
;
/*@=protoparamname@*/

#endif

